/*

This code was written by Kevin King for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Plumduino information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#ifndef PLUMDUINO_HARDWARE_H
#define PLUMDUINO_HARDWARE_H

#include "Arduino.h"
#include <FastLED.h>
#include <Adafruit_NeoPixel.h>
#include <EEPROM.h>


// ***************************************************
// Pin defines
// ***************************************************

#define SenseAmbient A2
#define SenseLeft A0
#define SenseRight A1
#define IRSource 5
#define Button 2
#define Light_Bus 4
#define LED 13        //orange general purpose LED on right side of board
#define PIN 4         //pixel pin, used for AdaFruit NeoPixel Libraray
#define DATA_PIN 4    //pixel pin, used for FastLED Library

//Pins for Plumduino Expansion Board
#define Slide1 A4
#define Slide2 A5
#define Chirp 9
#define Mic A3
#define Motion 8

// ***************************************************
// end Pin defines
// ***************************************************



// ***************************************************
// General hardware
// ***************************************************
#define SERIAL_SPEED 57600    //serial speed used for Serial Monitor window
#define NUM_PIXELS  8         //used for Adafruit_NeoPixel library
#define NUM_LEDS    8         //used for the FastLED library 
#define PROXFEEDBACK 35       //the amount of feedback to the proximity sensors. 35 is about normal.
#define MIC_CENTER 360        //center "quiet" reading of microphone

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUM_PIXELS, Light_Bus, NEO_GRB + NEO_KHZ800);  //start traditional NeoPixel

extern void hardwareBegin(void);

// ***************************************************
// end General hardware
// ***************************************************



// ***************************************************
// Plumduino Global Variables Initialization
// ***************************************************

int proxLeft, proxRight;    //proximity sensor readings
int red,green,blue;         //for results of getColorWheel() function
CRGB leds[NUM_LEDS];        //array to hold FastLED pixel data


// ***************************************************
// End of Plumduino Global Variables Initialization
// ***************************************************


// ***************************************************
// EEPROM Functions
// ***************************************************

//Copied from http://playground.arduino.cc/Code/EEPROMWriteAnything  Visit page for use instructions.
#include <EEPROM.h>
#include <Arduino.h>  // for type definitions

template <class T> int EEPROM_writeAnything(int ee, const T& value)
{
    const byte* p = (const byte*)(const void*)&value;
    unsigned int i;
    for (i = 0; i < sizeof(value); i++)
          EEPROM.write(ee++, *p++);
    return i;
}

template <class T> int EEPROM_readAnything(int ee, T& value)
{
    byte* p = (byte*)(void*)&value;
    unsigned int i;
    for (i = 0; i < sizeof(value); i++)
          *p++ = EEPROM.read(ee++);
    return i;
}

// ***************************************************
// end EEPROM Functions
// ***************************************************

#endif
