/*

This code was written by Kevin King for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Plumduino information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#include "PlumduinoHardware.h"
#include <FastLED.h>
#include <Adafruit_NeoPixel.h>
#include <EEPROM.h>

// ***************************************************
// Plumduino Hardware Initialization
// ***************************************************

void hardwareBegin(void){//Ver. 1.0, Kevin King

  pinMode(IRSource, OUTPUT);
  digitalWrite(IRSource, LOW);
  pinMode(SenseLeft, INPUT);
  pinMode(SenseRight, INPUT);
  pinMode(SenseAmbient, INPUT);
  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);
  Serial.begin(SERIAL_SPEED);
   
  FastLED.addLeds<NEOPIXEL, DATA_PIN>(leds, NUM_LEDS);  //start FastLED
                                                        //FastLED Tutorial: https://github.com/FastLED/FastLED/wiki/Basic-usage
                                                        //FastLED Docs: http://fastled.io/docs/3.1/class_c_fast_l_e_d.html
  FastLED.clear();
  FastLED.show();
}

// ***************************************************
// End of Plumduino Hardware Initialization
// ***************************************************



char buttonPressed(void){//Ver. 1.0, Dustin Soodak
  return (digitalRead(Button)==1);  // returns 1 (or "true") if button is being pressed. 
                                    // returns 0 (or "false") if not pressed.
}

void waitForButton(void){  // Ver 1.0, Kevin King
  while(!buttonPressed()){          // waits for button to be pressed
    //do nothing (while waiting for button press)
  }
}

void playChirp(unsigned int Frequency){//Ver. 1.0, Dustin Soodak
  //Frequency is in Hz
  //Amplitude is in units of UsOn
  int Amplitude = 50;
  uint16_t temp;
  uint16_t period,dutycycle;
  uint8_t prescalerbits;
  if(Amplitude>100)
    Amplitude=100;
  if (F_CPU/Frequency/2>=0x10000){
    if(F_CPU/Frequency/2>=0x10000*8){
        prescalerbits=0b011;//prescaler 64
        period=F_CPU/Frequency/(2*16);
        dutycycle=F_CPU/1000000*Amplitude/2/64;               
    }
    else{
      prescalerbits = 0b010;// prescaler 8
      period=F_CPU/Frequency/(2*8);
      dutycycle=F_CPU/1000000*Amplitude/2/8;
    }
  }
  else{
    prescalerbits = 0b001;  //on but no prescaling
    period=F_CPU/Frequency/(2*1);
    dutycycle=F_CPU/1000000*Amplitude/2/1;
  }
  if(Amplitude==0){
    TCCR1A=0;
    TCCR1B=0;
    pinMode(Chirp,INPUT);
    
    //Serial.println("off");    // debugging code
  }
  else{
    TCCR1B&=~0b00000111;//turn off timer
    ICR1=period;
    OCR1A=dutycycle;
    TCCR1A = (0b10<<6) | 0b10;//COM1A1 COM1A0, and WGM11 WGM10
    TCCR1B = (0b10<<3) | prescalerbits;//WGM13 WGM12, and off/(on with prescaler)
    pinMode(Chirp,OUTPUT);//DDRB|=1;//make PortB pin1 an output 
  }
}

void offChirp(void){//Ver. 1.0, Kevin King              //stops chirp tone
  noTone(Chirp);
  //PlayChirp(0);       //turn off chirp
}

void setPixelRGB(int Pixel, int Red, int Green, int Blue){//Ver. 1.0, Dustin Soodak
  //if(Pixel>7)
  //  Pixel=1;
  //if(Pixel<0)
  //  Pixel=0;
  pixels.setPixelColor(Pixel, pixels.Color(Red,Green,Blue));
  pixels.show();
}

void setAllPixelsRGB(int Red, int Green, int Blue){//Ver. 1.1, Dustin Soodak
  char i;
  for(i=0;i<8;i++){
    pixels.setPixelColor(i, pixels.Color(Red,Green,Blue)); 
  }
  pixels.show();//added Ver.1.1
}

void pixelRed(int pixel, byte brightness){
  setPixelRGB(pixel,brightness,0,0);
}

void pixelGreen(int pixel, byte brightness){
  setPixelRGB(pixel,0,brightness,0);
}

void pixelBlue(int pixel, byte brightness){
  setPixelRGB(pixel,0,0,brightness);
}

void pixelPurple(int pixel, byte brightness){
  byte n;
  n=brightness/2;
  setPixelRGB(pixel,n,0,brightness);
}

void pixelPink(int pixel, byte brightness){
  byte n;
  n=brightness/2;
  setPixelRGB(pixel,brightness,0,n);
}

void pixelYellow(int pixel, byte brightness){
  setPixelRGB(pixel,brightness,brightness,0);
}

void pixelOrange(int pixel, byte brightness){
  byte n;
  n=brightness/4;
  setPixelRGB(pixel,brightness,n,0);
}

void pixelCyan(int pixel, byte brightness){
  setPixelRGB(pixel,0,brightness,brightness);
}

void pixelWhite(int pixel, byte brightness){
  setPixelRGB(pixel,brightness,brightness,brightness);
}


void offPixels(void){
  clearStrip();
}

void clearStrip() {
  for( int i = 0; i<NUM_PIXELS; i++){
    setPixelRGB(i,0,0,0);
    //pixels.setPixelColor(i, 0x000000); pixels.show();
  }
  //pixels.show();
}

int micAmplitude(void){   //returns amplitude of Mic signal
  int amp;
  amp = analogRead(Mic);
  amp = amp - MIC_CENTER;
  amp = abs(amp);
  return amp;
}

void readProximity(void){
  digitalWrite(IRSource, HIGH);
  delay(2);
  proxLeft = analogRead(SenseLeft);
  proxRight = analogRead(SenseRight);
  digitalWrite(IRSource, LOW);
  delay(2);
  proxLeft = proxLeft - analogRead(SenseLeft) - PROXFEEDBACK;
  proxRight = proxRight - analogRead(SenseRight) - PROXFEEDBACK;
  proxLeft = constrain(proxLeft, 0, 1024);
  proxRight = constrain(proxRight, 0, 1024);
}

void tab(void){
  Serial.print("\t");
}

void nl(void){
  Serial.println("");
}


//Convert a given hue and brightness to RGB(Red Green Blue) and populate the
//global variables red, green, and blue with the result which can be directly
//sent to the pixels via setPixelRGB() and the like.
//This function will automatically "unwrap" full rotations.
//  h is hue value, integer between 0 and 360
//  bright is the brightness value between 0 and 255
//  Based on code by Chris Hulbert, found at: http://splinter.com.au/blog/?p=29
void getColorWheel(int h, double bright) {

  //make sure value passed falls within 0 and 360
  if (h > 359){
    while (h > 359){
      h = h - 359;  // subtract full rotations until within range
    }
  }

  if (h < 0){
    while (h < 0){
      h = h + 359;  // add full rotations until within range
    }
    h = 359 - h;
  }

  //this is the algorithm to convert from RGB to HSV
  double r=0; 
  double g=0; 
  double b=0;
  double s=1;
  double v = bright / 255;

  double hf=h/60.0;

  int i=(int)floor(h/60.0);
  double f = h/60.0 - i;
  double pv = v * (1 - s);
  double qv = v * (1 - s*f);
  double tv = v * (1 - s * (1 - f));

  switch (i)
  {
  case 0: //rojo dominante
    r = v;
    g = tv;
    b = pv;
    break;
  case 1: //verde
    r = qv;
    g = v;
    b = pv;
    break;
  case 2: 
    r = pv;
    g = v;
    b = tv;
    break;
  case 3: //azul
    r = pv;
    g = qv;
    b = v;
    break;
  case 4:
    r = tv;
    g = pv;
    b = v;
    break;
  case 5: //rojo
    r = v;
    g = pv;
    b = qv;
    break;
  }

  //set each component to a integer value between 0 and 255
  red=constrain((int)255*r,0,255);
  green=constrain((int)255*g,0,255);
  blue=constrain((int)255*b,0,255);

}

