/*

Plumduino
Plumduino_BaseSketch_Rev01_01
Version 1.1 08/2016

This is a basic sketch that can be used as a starting point
for various functionality of the Plumduino board.

This code was written by Kevin King for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Plumduino information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#include "PlumduinoHardware.h"  // Leave this line first. Do not edit this line. This causes Arduino
                                // to include background functions when turning your code into
                                // machine language Plumduino can understand.


//These values set the overall operation parameters of the code

#define ABS_MAX_BRIGHTNESS 200  //absolute max brightness the code will allow the pixels to reach
#define ABS_MIN_BRIGHTNESS 20   //brightness when at minimum (as in a dark room)
#define DIM_INTERVAL 30         //seconds between light sensor evaluation & auto dim decrements

#define MAX_UPDATES_PER_SECOND 70 //fastest the sequence will update (appears to move faster)
#define MIN_UPDATES_PER_SECOND 20 //slowest the sequence will update (appears to move more slowly)


//These variables are adjusted between examples

uint8_t maxBrightness, presentBrightness; 

uint8_t lightPatternIndex = 0;
int updatesPerSecond = 50;
bool fadeDirection = 1;
bool EEPROM_NeedsSavedFlag = 0;

unsigned long lastTime, lastChange;


#define PATTERN_COUNT 8

//These objects are used by the FastLED library
CRGBPalette16 currentPalette;
TBlendType    currentBlending;

void setup(){
  hardwareBegin();       //initialize processor to work with the Plumduino hardware
  
  FastLED.clear();
  FastLED.show();
  delay(50);
  
  presentBrightness = 0;
  adjustBrightness();
  
  EEPROM_readAnything(1020, lightPatternIndex);  //read previous saved pattern index from EEPROM
  if (lightPatternIndex > PATTERN_COUNT){
    lightPatternIndex = 0;
  }
  updatePatternSelection();                     //set pattern selection to present index
 
  lastTime = millis();   //mark time
}



void loop() {

    if (buttonPressed()){
      lightPatternIndex++;
      if (lightPatternIndex > PATTERN_COUNT-1){
        lightPatternIndex = 0;
      }
      FastLED.clear();
      leds[lightPatternIndex] = CRGB::White;
      FastLED.show();
      updatePatternSelection();
      delay(10);    //debounce button
      lastChange = millis();
      EEPROM_NeedsSavedFlag = 1; //set flag to update EEPROM
      while(buttonPressed()){
        //do nothing (wait for button to be released)
      }
      delay(10);    //debounce button
    }

    // dim out LEDs temporarily and measure ambient room brightness
    if (millis() > lastTime+(DIM_INTERVAL * 1000)){
      adjustBrightness();
      lastTime = millis();
    }

    //Store present sequence to EEPROM if running for more than threshold time limit
    if ((millis() > lastChange+30000) && (EEPROM_NeedsSavedFlag == 1)){
      //update EEPROM - store present value for lightPatternIndex to EEPROM, blink light
      EEPROM_writeAnything(1020, lightPatternIndex);
      EEPROM_NeedsSavedFlag = 0;  //clear saved flag
      for(int i=0; i<4; i++){
        digitalWrite(LED, HIGH);
        delay(150);
        digitalWrite(LED, LOW);
        delay(150);
      }
    }

    // the rest of the code below is what actually animates the colors    
    static uint8_t startIndex = 0;
    startIndex = startIndex + 1; /* motion speed */
    
    setLEDdirection();  //set fade direction based on proximity sensors
    
    if (fadeDirection){
      LEDsFromPaletteColorsLeftRight(startIndex);
    }
    else{
      LEDsFromPaletteColorsRightLeft(startIndex);
    }
    
    FastLED.show();
    FastLED.delay(1000 / updatesPerSecond);
  
}


// The updatePatternSelection() function sets which palette of colors is used
void updatePatternSelection(void){
    switch (lightPatternIndex){
    case 0:
      currentBlending = LINEARBLEND;
      currentPalette = RainbowColors_p;
    break;
    case 1:
      PurpleGreenPalette();
      currentBlending = LINEARBLEND;
    break;
    case 2:
      PinkPurplePalette();
      currentBlending = LINEARBLEND;
    break;
    case 3:
      currentPalette = CloudColors_p;
      currentBlending = LINEARBLEND;
    break;
    case 4:
      PinkBluePalette();
      currentBlending = LINEARBLEND;
    break;
    case 5:
      currentPalette = ForestColors_p;
      currentBlending = LINEARBLEND;
    break;
    case 6:
      currentPalette = LavaColors_p;
      currentBlending = LINEARBLEND;
    break;
    case 7:
      LimePalette();
      currentBlending = LINEARBLEND;
    break;
    default:
      currentPalette = RainbowColors_p;
      currentBlending = LINEARBLEND;
    break;
  }
}

// These are custom color pallets. Change them or add to them, then adust the function names used
// under each "case" statement above to set that new color palette to the given "case" index position.
void PurpleGreenPalette()
{
    CRGB purple = CHSV( HUE_PURPLE, 255, 255);
    CRGB green  = CHSV( HUE_GREEN, 255, 255);
    CRGB aqua  = CHSV( HUE_AQUA, 255, 255);
    CRGB black  = CRGB::Black;
    
    currentPalette = CRGBPalette16(
                                   green,  green,  purple,  purple,
                                   purple, purple, aqua,  aqua,
                                   green,  purple,  aqua,  aqua,
                                   purple, aqua, purple,  aqua );
}


void PinkPurplePalette()
{
    CRGB purple = CHSV( HUE_PURPLE, 255, 255);
    CRGB pink  = CHSV( HUE_PINK, 255, 255);
    CRGB black  = CRGB::Black;
    
    currentPalette = CRGBPalette16(
                                   purple,  pink,  purple,  pink,
                                   purple,  pink,  purple,  pink,
                                   purple,  pink,  purple,  pink,
                                   pink,  pink,  purple,  purple );
}

void PinkBluePalette()
{
    CRGB purple = CHSV( HUE_PURPLE, 255, 255);
    CRGB pink  = CHSV( HUE_PINK, 255, 255);
    CRGB blue  = CHSV( HUE_BLUE, 255, 255);
    CRGB black  = CRGB::Black;
    
    currentPalette = CRGBPalette16(
                                   pink,  black,  blue,  black,
                                   pink,  pink,  blue,  black,
                                   blue,  black,  blue,  pink,
                                   blue,  blue,  pink,  black );
}

void LimePalette()
{
    CRGB green = CHSV( HUE_GREEN, 255, 255);
    CRGB yellow  = CHSV( HUE_YELLOW, 255, 255);
    CRGB yellowgreen  = CHSV( 75, 255, 255);
    CRGB black  = CRGB::Black;
    
    currentPalette = CRGBPalette16(
                                   green,  yellow,  yellowgreen,  green,
                                   yellowgreen,  black,  yellow,  green,
                                   green,  yellowgreen,  yellowgreen,  yellow,
                                   green,  yellowgreen,  yellow,  yellow );
}


void LEDsFromPaletteColorsLeftRight( uint8_t colorIndex)
{
    uint8_t brightness = 255;
    
    for( int i = 0; i < NUM_LEDS; i++) {
        leds[i] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
        colorIndex += 3;
    }
}

void LEDsFromPaletteColorsRightLeft( uint8_t colorIndex)
{
    uint8_t brightness = 255;
    
    for( int i = NUM_LEDS; i >= 0; i--) {
        leds[i] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
        colorIndex += 3;
    }
}


// The setLEDdirection() function will set the swoosh to go left-->right or left<--right based
// on the proximity sensors. Wave a hand over the proximity sensor on the opposite side to change
// direction of the swoosh
void setLEDdirection(void){
  readProximity();
  if (proxLeft > proxRight+200){
     fadeDirection = 1;  //fade left to right
  }
  if (proxRight > proxLeft+200){
     fadeDirection = 0;  //fade right to left
  }
}


// The adjustBrightness() function will fade down the LEDs, take an ambient light reading,
// base the brighness of the LEDs based on the ambient level, as well as the update speed
// (how fast it appears the swooshes move), it will then fade LEDs back to previous condition.
void adjustBrightness(void){  //ver 1.0, Kevin King

  int fadeIndex;
  int presentAmbient;
  
  fadeIndex = presentBrightness;

  // un-commnet the next 5 lines to create a fade-down effect prior to reading ambient light sensor
  //  for (fadeIndex = presentBrightness; fadeIndex > 5; fadeIndex = fadeIndex-2){
  //    FastLED.setBrightness(fadeIndex);
  //    FastLED.show();
  //    delay(30);    //adjust this value to control how quickly LEDs fade down prior to ambient light reading
  //  }

  // fade complete. now make sure all LEDs are off
  FastLED.setBrightness(0);
  FastLED.show();
  delay(5);      //short delay to make sure they all update to the off position

  // get ambient light level
  presentAmbient = analogRead(SenseAmbient);
  //Serial.print(presentAmbient);Serial.print("\t");  //uncomment this line for serial debugging

  // make sure our brightness stays within the defined limits at top of the sketch
  if (presentAmbient+5 < presentBrightness){
    presentBrightness = presentBrightness - 5;
    if(presentBrightness < ABS_MIN_BRIGHTNESS){
      presentBrightness = ABS_MIN_BRIGHTNESS;
    }
  }
  else{
    presentBrightness = min(presentAmbient, ABS_MAX_BRIGHTNESS);  // make sure brightness never more than ABS_MAX
  }

  //Serial.println(presentBrightness);//Serial.print("\t"); //uncomment this line for serial debugging

  // updates per second is based on brighness
  updatesPerSecond = presentBrightness * 0.8;

  // un-commnet the next 5 lines to create a fade-up effect after reading ambient light sensor
  //  for (fadeIndex = 5; fadeIndex < presentBrightness; fadeIndex = fadeIndex+2){
  //    FastLED.setBrightness(fadeIndex);
  //    FastLED.show();
  //    delay(30);    //adjust this value to control how quickly the lights fade back up after ambient reading
  //  }
  
  FastLED.setBrightness(presentBrightness);
  FastLED.show();
}



